#!/usr/bin/python
# -*-coding:utf-8-*-

"""
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions: 
* 
* The above notice and this permission notice shall be included in all copies
* or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* File for "Fog" lesson of the OpenGL tutorial on
* www.videotutorialsrock.com
*
* File in python created by Sehnoax based on files from 
* www.videotutorialsrock.com
"""

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from wx import *
from wx.glcanvas import *


BOX_SIZE = 7.


class myGLCanvas(GLCanvas):
    def __init__(self, parent):
        """
        Création du canevas recevant la modélisation 3D
        """
        GLCanvas.__init__(self, parent,-1)

        self.init = False
        self._angle = 30.
        self.context = GLContext(self)

        glutInit()
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH)
    
        self.Bind(EVT_PAINT, self.OnPaint)
        self.Bind(EVT_SIZE, self.OnResize)
        self.Bind(EVT_KEY_DOWN, self.handleKeypress)


    def initRendering(self):
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_COLOR_MATERIAL)
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)
        glEnable(GL_LIGHT1)
        glEnable(GL_NORMALIZE)
        glEnable(GL_FOG)
        glClearColor(0.5, 0.5, 0.5, 1)


    def OnPaint(self, event):
        if not self.init :
            self.init = True

            self.SetCurrent(self.context)
            self.initRendering()

            self.loadTexture()           
            self.update(0)
            self.OnResize(0)
        
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()

        fogColor = [0.5, 0.5, 0.5, 1]
        glFogfv(GL_FOG_COLOR, fogColor)
        glFogi(GL_FOG_MODE, GL_LINEAR)
        glFogf(GL_FOG_START, 10.0)
        glFogf(GL_FOG_END, 20.0)
        glFogf(GL_FOG_DENSITY, 0.05)
        
        glTranslatef(0.0, 0.0, -20.0)

        ambientLight = [0.3, 0.3, 0.3, 1.0]
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight)
        
        directedLight = [0.7, 0.7, 0.7, 1.0]
        directedLightPos = [-2*BOX_SIZE, 4*BOX_SIZE, 1.0]
        glLightfv(GL_LIGHT0, GL_DIFFUSE, directedLight)
        glLightfv(GL_LIGHT0, GL_POSITION, directedLightPos)
                
        glRotatef(-self._angle, 1.0, 1.0, 0.0)

        glBegin(GL_QUADS)
        
        glColor3f(1.0, 1.0, 0.0)
        glNormal3f(0.0, 1.0, 0.0)
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)

        glColor3f(1.0, 0.0, 1.0)
        glNormal3f(0.0, -1.0, 0.0)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)

        glNormal3f(-1.0, 0.0, 0.0)
        glColor3f(0.0, 1.0, 1.0)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)
        glColor3f(0.0, 0.0, 1.0)        
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)

        glNormal3f(1.0, 0.0, 0.0)
        glColor3f(1.0, 0.0, 0.0)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)
        glColor3f(0.0, 1.0, 0.0)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)
        
        glEnd()

        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        glColor3f(1.0, 1.0, 1.0)


        glBegin(GL_QUADS)
        
        glNormal3f(0.0, 0.0, 1.0)
        glTexCoord2f(0.0, 0.0)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)
        glTexCoord2f(1.0, 0.0)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, BOX_SIZE/2)
        glTexCoord2f(1.0, 1.0)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)
        glTexCoord2f(0.0, 1.0)
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, BOX_SIZE/2)

        glNormal3f(0.0, 0.0, -1.0)
        glTexCoord2f(0.0, 0.0)
        glVertex3f(-BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        glTexCoord2f(1.0, 0.0)
        glVertex3f(-BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)
        glTexCoord2f(1.0, 1.0)
        glVertex3f(BOX_SIZE/2, BOX_SIZE/2, -BOX_SIZE/2)
        glTexCoord2f(0.0, 1.0)
        glVertex3f(BOX_SIZE/2, -BOX_SIZE/2, -BOX_SIZE/2)
        
        glEnd()

        glDisable(GL_TEXTURE_2D)
	
        self.SwapBuffers()
           

    def OnResize(self, event):
        if not self.init : return
        
        w, h = self.GetSize()

        glViewport(0, 0, w, h)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        gluPerspective(45., \
                       float(w)/float(h), \
                       1., \
                       200.)

        self.Refresh()


    def update(self, value):
        self._angle += 1.
        if self._angle > 360. : self._angle -= 360.
        
        self.OnPaint(1)
        CallLater(25, self.update, 0)

        
    def handleKeypress(self, event):
        key = event.GetUnicodeKey()
        if key == 27 : exit()


    def loadTexture(self):
        self.textureId = glGenTextures(1)        

        try:
            image = Image("vtr.bmp", wx.BITMAP_TYPE_ANY)       
            w, h = image.GetSize()
        except:
            return
 
        pixels = image.Mirror().Rotate180().GetData()  
                
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        
        glTexImage2D(GL_TEXTURE_2D, \
                     0, \
                     GL_RGB, \
                     w, h, \
                     0, \
                     GL_RGB, \
                     GL_UNSIGNED_BYTE, \
                     pixels)
        
        del image
        del pixels
            
        
class test(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, \
                          title = "Basic Shapes - videotutorialsrock.com", \
                          size = (506, 432))
        aire = myGLCanvas(self)
        panel = wx.Panel(self, -1, size = (100, 432))

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(aire, 1, wx.EXPAND)
        sizer.Add(panel, 0)
        
        self.SetSizer(sizer)

        self.Show()


if __name__ == '__main__':
    app = wx.App()
    frame = test()
    frame.Show()
    app.MainLoop()
